@@ -5,8 +5,11 @@ use strict;
 use warnings;
 use Carp;
 use Time::Local qw//;
+use POSIX qw//;
+use base qw/Exporter/;
 
-our $VERSION = "0.02";
+our $VERSION = "0.10";
+our @EXPORT_OK = qw/strftime/;
 
 use constant {
     SEC => 0,
@@ -21,20 +24,25 @@ use constant {
     ISO_WEEK_START_WDAY => 1,  # Monday
     ISO_WEEK1_WDAY      => 4,  # Thursday
     YDAY_MINIMUM        => -366,
-    FMT => 0,
-    ARGS => 1,
-    TZOFFSET => 2,
-    TZNAME => 3,
-    ISDST_CACHE => 4,
-    CODE => 6,
 };
 
-our %formats = (
-    'rfc2822' => '%a, %d %b %Y %T %z',
-    'rfc822' => '%a, %d %b %y %T %z',
-);
+BEGIN {
+    *tzoffset = \&_tzoffset;
+    *tzname = \&_tzname;
 
-# copy from POSIX/strftime/GNU/PP.pm
+    if (eval { require POSIX::strftime::GNU::XS; 1; }) {
+        no warnings 'redefine';
+        if ( POSIX::strftime::GNU::XS::strftime('%z', localtime) =~ /^[+-]\d{4}$/) {
+            *tzoffset = sub { POSIX::strftime::GNU::XS::strftime("%z", @_) };
+        }
+        if ( length POSIX::strftime::GNU::XS::strftime('%Z', localtime) ) {
+            *tzname = sub { POSIX::strftime::GNU::XS::strftime("%Z", @_) };
+        }
+    }
+}
+
+
+# copy from POSIX/strftime/GNU/PP.pm and modify
 my @offset2zone = qw(
     -11       0 SST     -11       0 SST
     -10       0 HAST    -09       1 HADT
@@ -102,53 +110,37 @@ my @offset2zone = qw(
     +14       1 WSDT    +13       0 WST
 );
 
-sub tzoffset {
-    my ($colons, @t) = @_;
+sub _tzoffset {
+    my @t = @_;
 
     # Normalize @t array, we need seconds without frac
-    $t[0] = int $t[0];
+    $t[SEC] = int $t[SEC];
 
     my $diff = (exists $ENV{TZ} and $ENV{TZ} eq 'GMT')
              ? 0
              : Time::Local::timegm(@t) - Time::Local::timelocal(@t);
-    my $h = $diff / 60 / 60;
-    my $m = $diff / 60 % 60;
-    my $s = $diff % 60;
-
-    my $fmt = do {
-        if ($colons == 0) {
-            '%+03d%02u';
-        }
-        elsif ($colons == 1) {
-            '%+03d:%02u';
-        }
-        elsif ($colons == 2) {
-            '%+03d:%02u:%02u';
-        }
-        elsif ($colons == 3) {
-            $s ? '%+03d:%02u:%02u' : $m ? '%+03d:%02u' : '%+03d';
-        }
-        else {
-            '%%' . ':' x $colons . 'z';
-        };
-    };
-
-    return sprintf $fmt, $h, $m, $s;
+    sprintf '%+03d%02u', $diff/60/60, $diff/60%60;
 }
 
-sub tzname {
+sub _tzname {
     my @t = @_;
 
     return 'GMT' if exists $ENV{TZ} and $ENV{TZ} eq 'GMT';
 
-    my $diff = tzoffset(3, @t);
+    my $diff = tzoffset(@t);
+    $diff =~ s!(\d\d)(\d\d)$!$1:$2!;
+    $diff =~ s!:00!!g;
 
     my @t1 = my @t2 = @t;
     @t1[3,4] = (1, 1);  # winter
     @t2[3,4] = (1, 7);  # summer
 
-    my $diff1 = tzoffset(3, @t1);
-    my $diff2 = tzoffset(3, @t2);
+    my $diff1 = tzoffset(@t1);
+    $diff1 =~ s!(\d\d)(\d\d)$!$1:$2!;
+    $diff1 =~ s!:00!!g;
+    my $diff2 = tzoffset(@t2);
+    $diff2 =~ s!(\d\d)(\d\d)$!$1:$2!;
+    $diff2 =~ s!:00!!g;
 
     for (my $i=0; $i < @offset2zone; $i += 6) {
         next unless $offset2zone[$i] eq $diff1 and $offset2zone[$i+3] eq $diff2;
@@ -180,7 +172,7 @@ sub isleap {
 sub isodaysnum {
     my @t = @_;
 
-    # Normalize @t array, we need WDAY
+    # Normalize @t array
     $t[SEC] = int $t[SEC];
 
     my $year = ($t[YEAR] + ($t[YEAR] < 0 ? 1900 % 400 : 1900 % 400 - 400));
@@ -214,12 +206,7 @@ sub isoweeknum {
     return int($days / 7) + 1;
 }
 
-sub first_day_wday {
-    my $year = shift;
-    (gmtime( Time::Local::timegm((0,0,0,1,0,$year)) ))[WDAY];
-}
-
-our %rules = (
+our %sprintf_rules = (
     '%' => [q!%s!, q!%!],
     'a' => [q!%s!, q!$weekday_abbr[$_[WDAY]]!],
     'A' => [q!%s!, q!$weekday_name[$_[WDAY]]!],
@@ -231,8 +218,6 @@ our %rules = (
     'D' => [q!%02d/%02d/%02d!, q!$_[MONTH]+1,$_[DAY],$_[YEAR]%100!],
     'e' => [q!%2d!, q!$_[DAY]!],
     'F' => [q!%04d-%02d-%02d!, q!$_[YEAR]+1900,$_[MONTH]+1,$_[DAY]!],
-    'G' => [q!%04d!, q!isoyearnum(@_)!],
-    'g' => [q!%02d!, q!isoyearnum(@_)%100!],
     'h' => [q!%s!, q!$month_abbr[$_[MONTH]]!],
     'H' => [q!%02d!, q!$_[HOUR]!],
     'I' => [q!%02d!, q!$_[HOUR]%12 || 1!],
@@ -247,84 +232,159 @@ our %rules = (
     'P' => [q!%s!, q!$_[HOUR] > 0 && $_[HOUR] < 13 ? "am" : "pm"!],
     'r' => [q!%02d:%02d:%02d %s!, q!$_[HOUR]%12 || 1, $_[MIN], $_[SEC], $_[HOUR] > 0 && $_[HOUR] < 13 ? "AM" : "PM"!],
     'R' => [q!%02d:%02d!, q!$_[HOUR], $_[MIN]!],
-    's' => [q!%s!, q!int(Time::Local::timegm(@_))!],
     'S' => [q!%02d!, q!$_[SEC]!],
     't' => [q!%s!, q!"\t"!],
     'T' => [q!%02d:%02d:%02d!, q!$_[HOUR], $_[MIN], $_[SEC]!],
     'u' => [q!%d!, q!$_[WDAY] || 7!],
-    'U' => [q!%02d!, q!int( ($_[YDAY] + 1 - (7 - first_day_wday($_[YEAR])) + 6 ) / 7 )!], #first Sunday as the first day of week 01
-    'V' => [q!%02d!, q!isoweeknum(@_)!],
     'w' => [q!%d!, q!$_[WDAY]!],
-    'W' => [q!%02d!, q!int( ($_[YDAY] + 1 - (7 - first_day_wday($_[YEAR])) + 5) / 7 )!], #first Monday as the first day of week 01
     'x' => [q!%02d/%02d/%02d!, q!$_[MONTH]+1,$_[DAY],$_[YEAR]%100!],
     'X' => [q!%02d:%02d:%02d!, q!$_[HOUR], $_[MIN], $_[SEC]!],
     'y' => [q!%02d!, q!$_[YEAR]%100!],
     'Y' => [q!%02d!, q!$_[YEAR]+1900!],
-    'z' => [q!%s!, q!$this->[TZOFFSET]!],
-    'Z' => [q!%s!, q!$this->[TZNAME]!],
     '%' => [q!%s!, q!'%'!],
 );
 
-my $char_handler = sub {
-    my ($self, $char) = @_;
-    die unless exists $rules{$char};
-    my ($format, $code) = @{$rules{$char}};
-    push @{$self->[ARGS]}, $code;
+our %rules = (
+    '%' => [q!'%'!],
+    'a' => [q!$weekday_abbr[$_[WDAY]]!,1],
+    'A' => [q!$weekday_name[$_[WDAY]]!,1],
+    'b' => [q!$month_abbr[$_[MONTH]]!],
+    'B' => [q!$month_name[$_[MONTH]]!],
+    'c' => [q!$weekday_abbr[$_[WDAY]] . ' ' . $month_abbr[$_[MONTH]] . ' ' . substr(' '.$_[DAY],-2) . ' %H:%M:%S %Y'!,1],
+    'C' => [q!substr('0'.int(($_[YEAR]+1900)/100), -2)!],  #century
+    'h' => [q!$month_abbr[$_[MONTH]]!],
+    'N' => [q!substr(sprintf('%.9f', $_[SEC] - int $_[SEC]), 2)!],
+    'n' => [q!"\n"!],
+    'p' => [q!$_[HOUR] > 0 && $_[HOUR] < 13 ? "AM" : "PM"!],
+    'P' => [q!$_[HOUR] > 0 && $_[HOUR] < 13 ? "am" : "pm"!],
+    'r' => [q!sprintf('%02d:%02d:%02d %s',$_[HOUR]%12 || 1, $_[MIN], $_[SEC], $_[HOUR] > 0 && $_[HOUR] < 13 ? "AM" : "PM")!],
+    't' => [q!"\t"!],
+    'x' => [q!'%m/%d/%y'!],
+    'X' => [q!'%H:%M:%S'!],
+    'z' => [q!'%z'!,1],
+);
+
+if ( $^O eq 'MSWin32' || $^O eq 'Cygwin' ) {
+    %rules = (
+        %rules,
+        'D' => [q!'%m/%d/%y'!],
+        'F' => [q!'%Y-%m-%d'!],
+        'G' => [q!substr('0000'. isoyearnum(@_), -4)!,1],
+        'R' => [q!'%H:%M'!],
+        'T' => [q!'%H:%M:%S'!],
+        'V' => [q!substr('0'.isoweeknum(@_),-2)!,1],
+        'e' => [q!substr(' '.$_[DAY],-2)!],
+        'g' => [q!substr('0'.isoyearnum(@_)%100,-2)!,1],
+        'k' => [q!substr(' '.$_[HOUR],-2)!],
+        'l' => [q!substr(' '.($_[HOUR]%12 || 1),-2)!],
+        's' => [q!int(Time::Local::timegm(@_))!,1],
+        'u' => [q!$_[WDAY] || 7!,1],
+        'z' => [q!tzoffset(@_)!,1],
+        'Z' => [q!tzname(@_)!,1],
+    );
+}
+
+my $sprintf_char_handler = sub {
+    my ($char,$args) = @_;
+    die unless exists $sprintf_rules{$char};
+    my ($format, $code) = @{$sprintf_rules{$char}};
+    push @$args, $code;
     return $format;
 };
 
-sub new {
-    my $class = shift;
-    my $fmt = shift || "rfc2822";
-    $fmt = $formats{$fmt} if exists $formats{$fmt};
-
-    my $self = bless [$fmt], $class;
-    $self->compile();
-    return $self;
-}
-
+my $char_handler = sub {
+    my ($char,$need9char_ref) = @_;
+    die unless exists $rules{$char};
+    my ($code,$flag) = @{$rules{$char}};
+    $$need9char_ref++ if $flag;
+    q|! . | . $code . q| . q!|;
+};
 
 sub compile {
-    my $self = shift;
-    my $fmt = $self->[FMT];
-    $self->[ARGS] = [];
-    my $rule_chars = join "", keys %rules;
+    my ($fmt) = @_;
+
+    my @weekday_name = qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday);
+    my @weekday_abbr = qw(Sun Mon Tue Wed Thu Fri Sat);
+    my @month_name = qw(January February March April May June July August September October November December);
+    my @month_abbr = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
+
+    $fmt =~ s/!/\\!/g;
     $fmt =~ s!\%E([cCxXyY])!%$1!g;
     $fmt =~ s!\%O([deHImMSuUVwWy])!%$1!g;
-    $fmt =~ s!
+
+    my $sprintf_fmt = $fmt;
+    my $disable_sprintf=0;
+    my $sprintf_code = '';
+    while ( $sprintf_fmt =~ m~ (?:\%([\%\+a-zA-Z])) ~gx ) {
+        if ( ! exists $sprintf_rules{$1} ) {
+            $disable_sprintf++
+        }
+    }
+    if ( !$disable_sprintf ) {
+        my $rule_chars = join "", keys %sprintf_rules;
+        my @args;
+        $sprintf_fmt =~ s!
+            (?:
+                 \%([$rule_chars])
+            )
+        ! $sprintf_char_handler->($1,\@args) !egx;
+        $sprintf_code = q~if ( @_ == 9 ) {
+            return sprintf(q!~ . $sprintf_fmt .  q~!,~ . join(",", @args) . q~);
+        }~;
+    }
+
+    my $posix_fmt = $fmt;
+    my $rule_chars = join "", keys %rules;
+    my $need9char=0;
+    $posix_fmt =~ s!
         (?:
              \%([$rule_chars])
         )
-    ! $char_handler->($self, $1) !egx;
-    my $args = $self->[ARGS];
-    my @weekday_name = qw( Sunday Monday Tuesday Wednesday Thursday Friday Saturday);
-    my @weekday_abbr = qw( Sun Mon Tue Wed Thu Fri Sat );
-    my @month_name = qw( January February March April May June July August September October November December );
-    my @month_abbr = qw( Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec );
-    $fmt = q~sub ($@) {
-        my $this = shift;
-        if ( @_ != 9  && @_ != 6 ) {
-            Carp::croak 'Usage: to_string(sec, min, hour, mday, mon, year, wday = -1, yday = -1, isdst = -1)';
-        }
-        if ( @_ == 6 ) {
-            @_ = gmtime(Time::Local::timegm(@_));
-        }
-        if ( ! defined $this->[TZOFFSET] || ! defined $this->[ISDST_CACHE] || $_[ISDST] ne $this->[ISDST_CACHE] ) {
-            $this->[ISDST_CACHE] = $_[ISDST];
-            $this->[TZOFFSET] = tzoffset(0, @_);
-            $this->[TZNAME] = tzname(@_);
-        }
-        sprintf(q!~ . $fmt .  q~!,~ . join(",", @$args) . q~);
-    }~;
-    $self->[CODE] = $fmt;
-    my $handler = eval $fmt; ## no critic    
-    die $@ if $@;
-    {
-        no warnings 'redefine';
-        *to_string = $handler;
+    ! $char_handler->($1,\$need9char) !egx;
+    
+    my $need9char_code='';
+    if ( $need9char ) {
+        $need9char_code = q~if ( @_ == 6 ) {
+          @_ = gmtime Time::Local::timegm(@_);
+        }~;
     }
+
+    my $code = q~sub {
+        if ( @_ != 9  && @_ != 6 ) {
+            Carp::croak 'Usage: strftime(sec, min, hour, mday, mon, year, wday = -1, yday = -1, isdst = -1)';
+        }
+        ~ . $sprintf_code . q~
+        ~ . $need9char_code . q~
+        POSIX::strftime(q!~ . $posix_fmt . q~!,@_);
+    }~;
+    my $sub = eval $code; ## no critic
+    die $@ ."\n===\n".$code if $@;
+    wantarray ? ($sub,$code) : $sub;
 }
 
+my %STRFTIME;
+sub strftime {
+    my $fmt = shift;
+    $STRFTIME{$fmt} ||= compile($fmt);
+    $STRFTIME{$fmt}->(@_);
+}
+
+sub new {
+    my $class = shift;
+    my $fmt = shift;
+    my ($sub,$code) = compile($fmt);
+    bless [$sub,$code], $class;
+}
+
+sub to_string {
+    my $self = shift;
+    $self->[0]->(@_);
+}
+
+sub code_ref {
+    my $self = shift;
+    $self->[0];
+}
 
 1;
 __END__
@@ -333,246 +393,64 @@ __END__
 
 =head1 NAME
 
-POSIX::strftime::Compiler - Compile strftime to perl. for loggers and servers
+POSIX::strftime::Compiler - GNU C library compatible strftime for loggers and servers
 
 =head1 SYNOPSIS
 
-    use POSIX::strftime::Compiler;
+    use POSIX::strftime::Compiler qw/strftime/;
 
-    my $psc = POSIX::strftime::Compiler->new('%a, %d %b %Y %T %z');
-    say $psc->to_string(localtime):
+    say strftime('%a, %d %b %Y %T %z',localtime):
+    
+    my $psc = POSIX::strftime::Compiler->new($fmt);
+    say $psc->to_string(localtime);
 
 =head1 DESCRIPTION
 
-POSIX::strftime::Compiler compiles strftime's format to perl. And generates formatted string.
-Because this module compiles strftime to perl code, it has good performance.
+POSIX::strftime::Compiler provides GNU C library compatible strftime(3). But this module will not affected
+by the system locale.  This feature is useful when you want to write loggers, servers and portable applications.
 
-POSIX::strftime::Compiler has compatibility with GNU's strftime, but this module will not 
-affected by the system locale. Because this module does not use strftime(3). 
-This feature is useful when you want to write loggers, servers and portable applications.
+For generate same result strings on any locale, POSIX::strftime::Compiler wraps POSIX::strftime and 
+converts some format characters to perl code
+
+=head1 FUNCTION
+
+=over 4
+
+=item strftime($fmt:String, @time)
+
+Generate formatted string from a format and time.
+
+  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
+  strftime('%d/%b/%Y:%T %z',$sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst):
+
+Compiled codes are stored in C<%POSIX::strftime::Compiler::STRFTIME>. This function is not exported by default.
+
+=back
 
 =head1 METHODS
 
 =over 4
 
-=item new($fmt:String)
+=item new($fmt)
 
-create instance of POSIX::strftime::Compiler.
+create instance of POSIX::strftime::Compiler
 
 =item to_string(@time)
 
-generate formatted string.
-
-  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
-  $psc->to_string($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst):
+Generate formatted string from time.
 
 =back
 
-=head1 SUPPORTED FORMAT
+=head1 FORMAT CHARACTERS
 
-=over 4
+POSIX::strftime::Compiler supports almost all characters that GNU strftime(3) supports. 
+But C<%E[cCxXyY]> and C<%O[deHImMSuUVwWy]> are not supported, just remove E and O prefix.
 
-=item C<%a>
+=head1 PERFORMANCE ISSUES ON WINDOWS
 
-The abbreviated weekday name according to the current locale.
-
-=item C<%A>
-
-The full weekday name according to the current locale.
-
-=item C<%b>
-
-The abbreviated month name according to the current locale.
-
-=item C<%B>
-
-The full month name according to the current locale.
-
-=item C<%c>
-
-The preferred date and time representation for the current locale.
-
-=item C<%C>
-
-The century number (year/100) as a 2-digit integer. (SU)
-
-=item C<%d>
-
-The day of the month as a decimal number (range 01 to 31).
-
-=item C<%D>
-
-Equivalent to C<%m/%d/%y>. (for Americans only: Americans should note that in
-other countries C<%d/%m/%y> is rather common. This means that in international
-context this format is ambiguous and should not be used.) (SU)
-
-=item C<%e>
-
-Like C<%d>, the day of the month as a decimal number, but a leading zero is
-replaced by a space. (SU)
-
-=item C<%E>
-
-Modifier: use alternative format, see below. (SU)
-
-=item C<%F>
-
-Equivalent to C<%Y-%m-%d> (the ISO 8601 date format). (C99)
-
-=item C<%G>
-
-The ISO 8601 week-based year with century as a decimal number. The
-4-digit year corresponding to the ISO week number (see C<%V>). This has the
-same format and value as %Y, except that if the ISO week number belongs to the
-previous or next year, that year is used instead. (TZ)
-
-=item C<%g>
-
-Like C<%G>, but without century, that is, with a 2-digit year (00-99). (TZ)
-
-=item C<%h>
-
-Equivalent to C<%b>. (SU)
-
-=item C<%H>
-
-The hour as a decimal number using a 24-hour clock (range 00 to 23).
-
-=item C<%I>
-
-The hour as a decimal number using a 12-hour clock (range 01 to 12).
-
-=item C<%j>
-
-The day of the year as a decimal number (range 001 to 366).
-
-=item C<%k>
-
-The hour (24-hour clock) as a decimal number (range 0 to 23); single digits
-are preceded by a blank. (See also C<%H>.) (TZ)
-
-=item C<%l>
-
-The hour (12-hour clock) as a decimal number (range 1 to 12); single digits
-are preceded by a blank. (See also C<%I>.) (TZ)
-
-=item C<%m>
-
-The month as a decimal number (range 01 to 12).
-
-=item C<%M>
-
-The minute as a decimal number (range 00 to 59).
-
-=item C<%n>
-
-A newline character. (SU)
-
-=item C<%N>
-
-Nanoseconds (range 000000000 to 999999999). It is a non-POSIX extension and
-outputs a nanoseconds if there is floating seconds argument.
-
-=item C<%O>
-
-Modifier: use alternative format, see below. (SU)
-
-=item C<%p>
-
-Either "AM" or "PM" according to the given time value, or the corresponding
-strings for the current locale. Noon is treated as "PM" and midnight as "AM".
-
-=item C<%P>
-
-Like C<%p> but in lowercase: "am" or "pm" or a corresponding string for the
-current locale. (GNU)
-
-=item C<%r>
-
-The time in a.m. or p.m. notation. In the POSIX locale this is equivalent to
-C<%I:%M:%S %p>. (SU)
-
-=item C<%R>
-
-The time in 24-hour notation (%H:%M). (SU) For a version including the
-seconds, see C<%T> below.
-
-=item C<%s>
-
-The number of seconds since the Epoch, 1970-01-01 00:00:00 +0000 (UTC). (TZ)
-
-=item C<%S>
-
-The second as a decimal number (range 00 to 60). (The range is up to 60 to
-allow for occasional leap seconds.)
-
-=item C<%t>
-
-A tab character. (SU)
-
-=item C<%T>
-
-The time in 24-hour notation (C<%H:%M:%S>). (SU)
-
-=item C<%u>
-
-The day of the week as a decimal, range 1 to 7, Monday being 1. See also
-C<%w>. (SU)
-
-=item C<%U>
-
-The week number of the current year as a decimal number, range 00 to 53,
-starting with the first Sunday as the first day of week 01. See also C<%V> and
-C<%W>.
-
-=item C<%V>
-
-The ISO 8601 week number of the current year as a decimal number,
-range 01 to 53, where week 1 is the first week that has at least 4 days in the
-new year. See also C<%U> and C<%W>. (SU)
-
-=item C<%w>
-
-The day of the week as a decimal, range 0 to 6, Sunday being 0. See also
-C<%u>.
-
-=item C<%W>
-
-The week number of the current year as a decimal number, range 00 to 53,
-starting with the first Monday as the first day of week 01.
-
-=item C<%x>
-
-The preferred date representation for the current locale without the time.
-
-=item C<%X>
-
-The preferred time representation for the current locale without the date.
-
-=item C<%y>
-
-The year as a decimal number without a century (range 00 to 99).
-
-=item C<%Y>
-
-The year as a decimal number including the century.
-
-=item C<%z>
-
-The C<+hhmm> or C<-hhmm> numeric timezone (that is, the hour and minute offset
-from UTC). (SU)
-
-=item C<%Z>
-
-The timezone or name or abbreviation.
-
-=item C<%%>
-
-A literal C<%> character.
-
-=back
-
-C<%E[cCxXyY]> and C<%O[deHImMSuUVwWy]> are not supported, just remove E and O prefix.
+Windows and Cygwin and some system may not support C<%z> and C<%Z>. For these system, 
+POSIX::strftime::Compiler calculate time zone offset and find zone name. This is not fast.
+If you need performance on Windows and Cygwin, please install L<POSIX::strftime::GNU>
 
 =head1 SEE ALSO
 
